uniform float zNear = 0.3;
uniform float zFar = 1500.0;
uniform vec3 lightPosition;

#if SHADOWQUALITY > 0
in float blockBrightness;

in vec4 shadowCoordsFar;
uniform sampler2D shadowMapFar;
uniform float shadowIntensity = 1;
uniform float shadowMapWidthInv;
uniform float shadowMapHeightInv;


#endif

#if SHADOWQUALITY > 1
in vec4 shadowCoordsNear;
uniform sampler2D shadowMapNear;
#endif

uniform float flatFogDensity;
uniform float flatFogStart;
uniform float viewDistance;

float linearDepth(float depthSample)
{
    depthSample = 2.0 * depthSample - 1.0;
    float zLinear = 2.0 * zNear / (zFar + zNear - depthSample * (zFar - zNear));
    return zLinear;
}

vec4 applyFog(vec4 rgbaPixel, float fogWeight) {
	return vec4(mix(rgbaPixel.rgb, rgbaFog.rgb, fogWeight), rgbaPixel.a);
}


float getBrightnessFromShadowMap() {
	#if SHADOWQUALITY > 0
	float totalFar = 0.0;
	if (shadowCoordsFar.z < 0.999 && shadowCoordsFar.w > 0) {
		for (int x = -1; x <= 1; x++) {
			for (int y = -1; y <= 1; y++) {
				float nearestLight = texture(shadowMapFar, shadowCoordsFar.xy + vec2(x * shadowMapWidthInv, y * shadowMapHeightInv)).r;
				if (shadowCoordsFar.z > nearestLight + 0.0015 - (SHADOWQUALITY-1)*0.0003) {
					totalFar += 1.0;
				}
			}
		}
	}

	totalFar /= 9.0;
	
	float b = 1.0 - shadowIntensity * totalFar * shadowCoordsFar.w * 0.5;
	#endif
	
	
	#if SHADOWQUALITY > 1
	float totalNear = 0.0;
	if (shadowCoordsNear.z < 0.999 && shadowCoordsNear.w > 0) {
		for (int x = -2; x <= 2; x++) {
			for (int y = -2; y <= 2; y++) {
				float nearestLight = texture(shadowMapNear, shadowCoordsNear.xy + vec2(x * shadowMapWidthInv, y * shadowMapHeightInv)).r;
				if (shadowCoordsNear.z > nearestLight + 0.0015 - (SHADOWQUALITY-1)*0.00037) {
					totalNear += 1.0;
				}
			}
		}
	}
	
	totalNear /= 25.0;
	b -=  shadowIntensity * totalNear * shadowCoordsNear.w * 0.5;
	#endif
	
	#if SHADOWQUALITY > 0
	b = clamp(b + blockBrightness, 0, 1);
	return b;
	#endif
	
	return 1.0;
}


float getBrightnessFromNormal(vec3 normal, float normalShadeIntensity, float minNormalShade) {

	// Option 2: Completely hides peter panning, but makes semi sunfacing block sides pretty dark
	float nb = max(minNormalShade, 0.5 + 0.5 * dot(normal, lightPosition));
	
	// Let's also define that diffuse light from the sky provides an additional brightness boost for up facing stuff
	// because the top side of blocks being darker than the sides is uncanny o__O
	nb = max(nb, dot(normalize(normal), vec3(0, 1, 0)) * 0.95);
	
	// Let's also lastly define that the north side is always brighter
	float northness = max(0, dot(vec3(0,0,-1), normal));
	nb += northness * 0.2;
	
	return mix(1, nb, normalShadeIntensity);
}



vec4 applyFogAndShadow(vec4 rgbaPixel, float fogWeight) {
	float b = getBrightnessFromShadowMap();
	rgbaPixel *= vec4(b, b, b, 1);
	
	return applyFog(rgbaPixel, fogWeight);
}	

vec4 applyFogAndShadowWithNormal(vec4 rgbaPixel, float fogWeight, vec3 normal, float normalShadeIntensity, float minNormalShade) {
	float b = getBrightnessFromShadowMap();
	float nb = getBrightnessFromNormal(normal, normalShadeIntensity, minNormalShade);
	
	b = min(b, nb);
	rgbaPixel *= vec4(b, b, b, 1);
	
	return applyFog(rgbaPixel, fogWeight);
}



float getFogLevel(float fogMin, float fogDensity, float worldPosY) {
	//float depth = min(250, gl_FragCoord.z);

	float depth = gl_FragCoord.z;
	float clampedDepth = min(250, gl_FragCoord.z);
	
	//float distanceFog = 0;//pow(depth / viewDistance, 4);
	float distanceFog = -0.1 + fogMin + 1 - 1 / exp(depth * fogDensity + max(flatFogDensity / (flatFogDensity < 0 ? 8 : 1) * depth * (-5*flatFogStart) / 100, 0));
	float flatFog = (1 - 1 / exp((worldPosY - flatFogStart) * flatFogDensity)) - max(0.2, (1 - depth));
	
	float val = max(
		distanceFog,
		flatFog
	);	
	
	return clamp(val, 0, 1);
}