#ifndef DYNLIGHTS
    #define DYNLIGHTS 0
#endif
#if SHADOWQUALITY > 0
out float blockBrightness;
#endif

out float glowLevel;

uniform float flatFogDensity;
uniform float flatFogStart;
uniform float viewDistance;


#if DYNLIGHTS > 0
uniform vec3[DYNLIGHTS] pointLights;
uniform vec3[DYNLIGHTS] pointLightColors;
uniform int pointLightQuantity;
#endif


vec4 applyLightWithoutPointLight(vec4 sunColor, vec4 blockColor, float bGlow) {
	float bSun = (sunColor.r + sunColor.g + sunColor.b)/3;
	float bBlock = (blockColor.r + blockColor.g + blockColor.b)/3;
	
	// 1. Mix colors according to their brightness (very bright light has more influence on the color)
	vec4 rgba = (2 * bSun * sunColor + bBlock * blockColor) / (2 * bSun + bBlock);
	
	// 2. Fix brightness
	rgba *= max(bGlow, max(bSun, bBlock)) / ((rgba.r + rgba.g + rgba.b) / 3);
	
#if SHADOWQUALITY > 0
	blockBrightness = clamp(max(bGlow, bBlock) - bSun/2, 0, 1);
#endif
	
	// 4. Always fully opaque
	rgba.a = 1;
	
	return rgba;
}

// sunColor = color of the ambient light, or the sun color really
// lightColor = rgb is block light, a is sun light brightness
// blockColor = AO + Biome Tint + Block Side Shading
vec4 applyLight(vec3 ambientColor, vec4 lightColor, vec4 blockColor, int renderFlags, vec4 worldPos) {
	
	float bGlow = glowLevel = (renderFlags & 0xff) / 256.0;
	float contrast = 1.2;
	if (((renderFlags >> 13) & 1) > 0) contrast = 1.15;
	

	vec3 blockLightColor = lightColor.rgb;
	vec3 sunLightColor = lightColor.a * ambientColor.rgb;

#if DYNLIGHTS == 0
	return applyLightWithoutPointLight(vec4(sunLightColor * blockColor.rgb,1), vec4(blockLightColor * blockColor.rgb,1), bGlow);
#else

	// Sun brightness
	float bSun = (sunLightColor.r + sunLightColor.g + sunLightColor.b)/3;
	// Block brightness
	float bBlock = (blockLightColor.r + blockLightColor.g + blockLightColor.b)/3;
	// Point light brightness
	float bPoint = 0;
	
	float bPointBrightSum = 0;
	vec3 pointColSum = vec3(0);
	for (int i = 0; i < pointLightQuantity; i++) {
		vec4 lightVec = -vec4(worldPos.x - pointLights[i].x, worldPos.y - pointLights[i].y, worldPos.z - pointLights[i].z, 1);
		vec3 color = pointLightColors[i];
		
		float dist = pow(1.35, length(lightVec));
		//float dist = length(lightVec);
		float bright = (color.r + color.g + color.b);
		float strength = min(bright/3, bright / dist);
		
		bPoint = max(bPoint, strength);
		bPointBrightSum += strength;
		
		pointColSum.r +=  color.r * strength;
		pointColSum.g +=  color.g * strength;
		pointColSum.b +=  color.b * strength;
	}


	if (bPointBrightSum > 0) {
		pointColSum.rgb /= max(1, bPointBrightSum);
	}
	
	// 1. Mix colors according to their brightness (very bright light has more influence on the color)
	vec3 rgba = (2 * bSun * sunLightColor + bBlock * blockLightColor + bPoint * pointColSum) / (2 * bSun + bBlock + bPoint);
	
	// 2. Fix brightness
	float bMax = max(bGlow, max(bPoint, max(bSun, bBlock)));
	
#if SHADOWQUALITY > 0
	blockBrightness = clamp(max(bGlow, max(bPoint, bBlock)) - bSun/2, 0, 1);
#endif
	
	
	rgba *= bMax / ((rgba.r + rgba.g + rgba.b) / 3);
	rgba *= blockColor.rgb;
	
	rgba *= 1 + bGlow/4;
	
	rgba *= contrast;
	
	return vec4(rgba, 1);
#endif
}



float getFogLevel(vec4 worldPos, float fogMin, float fogDensity) {
	float depth = length(worldPos);
	float clampedDepth = min(250, depth);
	
	//float distanceFog = pow(depth / viewDistance, 4);
	float distanceFog = -0.1 + fogMin + 1 - 1 / exp(clampedDepth * fogDensity + max(-flatFogDensity * clampedDepth * (flatFogStart) / 160, 0));
	float flatFog = 1 - 1 / exp((worldPos.y - flatFogStart) * flatFogDensity) - clamp((2.5 / pow(1.25, clampedDepth / 1)), 0, 0.85);  // (the second part is for removing fog very close to the player)
	
	float val = max(flatFog, distanceFog);
	return clamp(val, 0, 1);
}



vec4 applyFog(vec4 worldPos, vec4 rgbaPixel, vec4 rgbaFog, float fogMin, float fogDensity) {
	float amount = getFogLevel(worldPos, fogMin, fogDensity);
	return vec4(mix(rgbaPixel.rgb, rgbaFog.rgb, amount), rgbaPixel.a * rgbaFog.a);
}



vec3 unpackNormal(int flags) {
	int x = (flags >> 1) & 0x7;
	int y = (flags >> 5) & 0x7;
	int z = (flags >> 9) & 0x7;
	
	int signx = flags & 1;
	int signy = (flags >> 4) & 1;
	int signz = (flags >> 8) & 1;
	
	return vec3(
		(1.0 - signx * 2) * x / 7.0,
		(1.0 - signy * 2) * y / 7.0,
		(1.0 - signz * 2) * z / 7.0
	);
}