#version 330 core

uniform sampler2D terrainTex;
uniform sampler2D depthTex;

uniform float blockTextureSize;
uniform float textureAtlasSize;
uniform float waterFlowCounter;
uniform vec2 frameSize;

in vec4 rgba;
in vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in vec2 uvSize;
in float waterStillCounterOff;
flat in vec2 uvBase;
in vec3 fragWorldPos;
in vec3 fragNormal;

in vec2 flowVectorf;
in float glowLevel;
flat in int waterFlags;
flat in int renderFoam;

layout(location = 0) out vec4 outAccu;
layout(location = 1) out vec4 outReveal;
layout(location = 2) out vec4 outGlow;

#include fogandlight.fsh
#include noise3d.ash

void drawPixel(vec4 color) {
    float alpha = color.a;

	float weight = color.a * clamp(0.03 / (1e-5 + pow(gl_FragCoord.z / 200, 4.0)), 1e-2, 1e3);
	
    // RGBA32F texture (accumulation)
    outAccu = vec4(color.rgb * color.a, color.a) * weight;

    // R32F texture (revealage)
    // Make sure to use the red channel (and GL_RED target in your texture)
    outReveal.r = alpha*1.2;
	
    outGlow = vec4(glowLevel, 0, 0, color.a);
}

void main() 
{
	vec4 texColor;

	vec2 uvInverse = uvBase + blockTextureSize - uvSize;

	if (length(flowVectorf) > 0.001) {
		vec2 flowVec = normalize(flowVectorf);
		float maxsize = blockTextureSize - 0.1f / textureAtlasSize;
		
		vec2 uvxOffset = mod((uv - uvBase) + flowVec * waterFlowCounter * blockTextureSize, vec2(maxsize));
		
		texColor = texture(terrainTex, uvBase + uvxOffset);
	} else {
		// Needs to be rewritten to not do weird uv-inverse math but simply use a second texture so json blocks can use it too
		texColor = texture(terrainTex, uv) * waterStillCounterOff + (1-waterStillCounterOff) * texture(terrainTex, uvInverse);				
	}
	
		texColor.a = 1;
		float bright = (rgba.r + rgba.g + rgba.b)/3;
		texColor = applyFogAndShadow(texColor * rgba, fogAmount);
		float x = gl_FragCoord.x / frameSize.x;
		float y = gl_FragCoord.y / frameSize.y;

	if (rgba.a > 0) {
	
		// Water edge + shinyness shading effect, kinda nice
		float ownDepth = linearDepth(gl_FragCoord.z);
		float diffTotal = 0;
		int range = 2;
		for (int dx = -range; dx <= range; dx++) {
			for (int dy = -range; dy <= range; dy++) {
				diffTotal += abs(ownDepth - linearDepth(texture(depthTex, vec2(x + dx/frameSize.x, y + dy/frameSize.y)).x));
			}
		}
		
		diffTotal /= (4*range * range);
		
		if ((waterFlags & 0x2000000) > 0) { // Bit 25
			// Lava
			float intensity = clamp(dot(fragNormal, vec3(0, 1, 0)), 0, 1) * 0.5;
			float a = fragWorldPos.x + fragWorldPos.y;
			float b = fragWorldPos.z;
			
			float diff = intensity * clamp(1 - diffTotal*1000 - cnoise(vec3(a*35, b*35, waterFlowCounter))/2 + cnoise(vec3(a*2, b*2, waterFlowCounter))/2, 0, 1);
			float noise = intensity * (cnoise(vec3(a, b, waterFlowCounter)) + 0.5) / 2;
			texColor.rgb -= vec3(2*bright*(diff * 0.3 + noise/10))/2;
			texColor.a += diff/16 + noise/4;
			
		} else {
			// Cold liquids
			
			float intensity = clamp(dot(fragNormal, vec3(0, 1, 0)), 0, 1);
			float a = fragWorldPos.x + fragWorldPos.y;
			float b = fragWorldPos.z;
			
			float diff = intensity * clamp(1 - diffTotal*4000 - cnoise(vec3(a*35, b*35, waterFlowCounter))/2 + cnoise(vec3(a*2, b*2, waterFlowCounter))/2, 0, 1);
			float noise = intensity * (cnoise(vec3(a, b, waterFlowCounter)) + 0.5) / 2;
			texColor.rgb += vec3(2*bright*(diff * 0.3 + noise/10));
			texColor.a += diff/16 + noise/4;
		}
	}
    
	drawPixel(texColor);
}